@@ -80,28 +80,31 @@ struct dav_node {
        information that the contents is no langer valid.
        So cache_path may be NULL.
        For directories: A file containing the directory entries.
-       For regular files: A local copy of the file. */
+       For regular files: A local copy of the file.
+          File times are set according the values from the server when the file
+          is downloaded. As long as the file is cached only dav_set_attr() may 
+          changed file attributes directly, but they may be changed by open,
+          read, write and close. */
     char *cache_path;
-    /* Files: Etag from the server; used for conditional GET requests. When a
-              file is opened for writing it is no longer updated untill the
-              file is put back to the server.
-       Directories: Unused. */
+    /* Etag from the server. Set when the resource is downloaded. Used for
+       conditional GET requests and to detect whether a resource has been
+       changed remotely. If present it overrides information from the
+       Last-Modified time (smtime). */
     char *etag;
     /* A linked list of handles for open files. */
     dav_handle *handles;
     /* Size of the contents of the node.
        Files: Initially set to the value of the content-lenght header. When
-              opened for write it is updated to the size of the cached file.
-       Directories: Size of the file containign the directory entries. */
+          the file is open it is updated to the size of the cached file.
+       Directories: Size of the file containing the directory entries. */
     off_t size;
-    /* Initially set to the value of mtime.
-       Files: When a local copy exists updated to atime of this.
-       Directories: Updated to system time by some upcalls. */
+    /* Initially set to the value of mtime. Updated by some upcalls
+       Files: When a local copy exists its atime is also used for update. */
     time_t atime;
     /* Initially set to the last-modified time from the  server, or the system
        time if created locally.
        Files: When a local cache file exists updated to mtime of this cache file. 
-       Used to evaluate whether a file is dirty. */
+          Used to evaluate whether a file is dirty. */
     time_t mtime;
     /* Initially set to the value of mtime. Updated by dav_setattr(). */
     time_t ctime;
@@ -109,7 +112,7 @@ struct dav_node {
               file is opened for writing. Used to check whether the file has
               been changed remotely. */
     time_t smtime;
-    /* Time when the node has to be updated from the server. */
+    /* Time when the node has last been updated from the server. */
     time_t utime;
     /* Files: Time when the lock expires.
               0 if not locked. -1 locked infinitely. */
@@ -122,6 +125,9 @@ struct dav_node {
         Note: Some server create an empty file when locking. Maybe the RFC will
               be changed this way. */
     int remote_exists;
+    /* Files: Dirty flag. Must be set, when a file is opened for writing, and
+              reset, when the file is saved back. */
+    int dirty;
     /* mode, uid and gid are initially set to the default values, but may be
        changed by dav_setattr(). */
     mode_t mode;
@@ -130,7 +136,7 @@ struct dav_node {
 };
 
 
-/* Returne by dav_statfs(). */
+/* Returned by dav_statfs(). */
 typedef struct dav_stat dav_stat;
 struct dav_stat {
     unsigned long long blocks;
@@ -146,10 +152,10 @@ struct dav_stat {
 /* Call back function that writes a directory entry to file descriptor fd.
    fd     : An open file descriptor to write to.
    off    : The current file size.
-   node   : The pointer if node is taken as inode/file number (shrinked to
+   node   : The pointer to node is taken as inode/file number (shrinked to
             fit if necessary).
    name   : File name; if NULL, the last, empty entry is written (if the
-            kernel file system wants one.
+            kernel file system wants one).
    return value : New size of the file. -1 in case of an error. */
 typedef off_t (*dav_write_dir_entry_fn)(int fd, off_t off, const dav_node *node,
                                         const char *name);
@@ -172,9 +178,9 @@ typedef off_t (*dav_write_dir_entry_fn)(int fd, off_t off, const dav_node *node,
    comes up.
    paramters: if not self explaining, please see mount_davfs.c, struct args. */
 void dav_init_cache(uid_t uid, gid_t gid, mode_t file_mode, mode_t dir_mode,
-                    mode_t file_umask, mode_t dir_umask, size_t h_table_size,
-                    time_t d_expire, time_t g_wait, time_t d_retry,
-                    time_t d_max_retry, time_t l_refresh, size_t cache_size,
+                    mode_t f_umask, mode_t d_umask, size_t hash_table_sz,
+                    time_t dir_rf, time_t file_rf, int gui_opt, time_t dir_rt,
+                    time_t dir_max_rt, time_t lock_rf, size_t cache_size,
                     const char *tl_cache_dir, const char *backup_dir,
                     const char *host, const char *path, const char *mpoint);
 
@@ -293,8 +299,8 @@ int dav_mkdir(dav_node **nodep, dav_node *parent, const char *name, uid_t uid,
 /* Opens file or directory node according to flags and returns file descriptor
    fd. fd, together with pid, pgid and uid, is stored in node for read, write
    and close operations.
-   Only the O_ACCESSMOE, O_APPEND, O_TRUNC and O_FSYNC bits in flags will be
-   honoured. O_CREATE and O_EXCL are not allowed.
+   Only the O_ACCESSMOE, O_APPEND and O_TRUNC bits in flags will be honoured.
+   O_CREATE and O_EXCL are not allowed.
    Permissions:
    uid must have execute permission for parent and all of its ancestors, as
    as well as read and/or write permission for node, according to the
@@ -306,7 +312,6 @@ int dav_open(int *fd, dav_node *node, int flags, pid_t pid, pid_t pgid,
    and copies them into buf.
    The number of bytes read is returned in len.
    The file must be opened readonly or readwrite. */
-/* TODO: Remove pid; check flags. */
 int dav_read(ssize_t *len, dav_node * node, int fd, char *buf, size_t size,
              off_t offset);
 
@@ -353,15 +358,14 @@ int dav_root(dav_node **nodep, uid_t uid);
    uid must have execute permission for parent directory of node and all of
    its ancestors.
    To change mode, owner, or gid, uid must be owner of the node or root.
-   To change atime, mtime, ctime or size, uid must have write permission for
+   To change atime, mtime or size, uid must have write permission for
    node.
    To change gid, uid must be member of the new group or root.
    Note: This attributes, except size and the execute bit of mode, are only
          changed locally and not on the server. */
 int dav_setattr(dav_node *node, uid_t uid, int sm, mode_t mode,
                 int so, uid_t owner, int sg, gid_t gid, int sat, time_t atime,
-                int smt, time_t mtime, int sct, time_t ctime,
-                int ssz, off_t size);
+                int smt, time_t mtime, int ssz, off_t size);
 
 
 /* Returns struct dav_stat which currently is mainly fake.
