@@ -37,6 +37,13 @@
 #include <time.h>
 #include <unistd.h>
 
+#ifdef ENABLE_NLS
+#   include <libintl.h>
+#   define _(String) gettext(String)
+#else
+#   define _(String) String
+#endif
+
 #include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
@@ -132,7 +139,7 @@ void dav_coda2_loop(int device, size_t bufsize, time_t idle_time,
     buf = malloc(BUF_SIZE);
     if (buf == NULL) {
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
-               "Could not allocate message buffer.");
+               _("can't allocate message buffer"));
         return;
     }
     static int flush = 0;
@@ -591,8 +598,6 @@ static uint32_t coda_setattr(void) {
                              in->attr.va_atime.tv_sec,
                              in->attr.va_mtime.tv_sec != -1,
                              in->attr.va_mtime.tv_sec,
-                             in->attr.va_ctime.tv_sec != -1,
-                             in->attr.va_ctime.tv_sec,
                              in->attr.va_size != UINT64_MAX,
                              in->attr.va_size);
 
