@@ -29,10 +29,14 @@
 
 #define DBG_ARGS do { \
     DBG0("  Arguments after parsing config files:"); \
-    DBG1("    dav_group = %i", args->dav_group); \
+    DBG1("    dav_user = %s", args->dav_user); \
+    DBG1("    dav_group = %s", args->dav_group); \
+    DBG1("    ignore_home = %s", args->ignore_home); \
     DBG1("    conf = %s", args->conf); \
     DBG1("    user = %i", args->user); \
+    DBG1("    netdev = %i", args->netdev); \
     DBG1("    mopts = %#lx", args->mopts); \
+    DBG1("    add_mopts = %s", args->add_mopts); \
     DBG1("    kernel_fs = %s", args->kernel_fs); \
     DBG1("    buf_size = %i KiBytes", args->buf_size); \
     DBG1("    uid = %i", args->uid); \
@@ -45,26 +49,32 @@
     DBG1("    host = %s", args->host); \
     DBG1("    port = %i", args->port); \
     DBG1("    path = %s", args->path); \
+    DBG1("    servercert = %s", args->servercert); \
     DBG1("    secrets = %s", args->secrets); \
+    DBG1("    clicert = %s", args->clicert); \
     DBG1("    p_host = %s", args->p_host); \
     DBG1("    p_port = %i", args->p_port); \
     DBG1("    useproxy = %i", args->useproxy); \
     DBG1("    askauth = %i", args->askauth); \
     DBG1("    displayname = %i", args->displayname); \
     DBG1("    locks = %i", args->locks); \
+    DBG1("    expect100 = %i", args->expect100); \
     DBG1("    read_timeout = %li s", args->read_timeout); \
+    DBG1("    server_charset = %s", args->s_charset); \
     DBG1("    sys_cache = %s", args->sys_cache); \
     DBG1("    cache_dir = %s", args->cache_dir); \
     DBG1("    backup_dir = %s", args->backup_dir); \
     DBG1("    cache_size = %i MiByte", args->cache_size); \
     DBG1("    table_size = %i", args->table_size); \
     DBG1("    idle_time = %li s", args->idle_time); \
-    DBG1("    expire = %li s", args->expire); \
-    DBG1("    get_wait = %li s", args->get_wait); \
+    DBG1("    dir_refresh = %li s", args->dir_refresh); \
+    DBG1("    file_refresh = %li s", args->file_refresh); \
+    DBG1("    gui_optimize = %i", args->gui_optimize); \
     DBG1("    retry = %li s", args->retry); \
     DBG1("    max_retry = %li s", args->max_retry); \
     DBG1("    lock_timeout = %li s", args->lock_timeout); \
     DBG1("    lock_refresh = %li s", args->lock_refresh); \
+    DBG1("    lock_owner = %s", args->lock_owner); \
 } while (0)
 
 #define DBG_CMDLINE(x) do { \
