@@ -35,6 +35,13 @@
 #include <time.h>
 #include <unistd.h>
 
+#ifdef ENABLE_NLS
+#   include <libintl.h>
+#   define _(String) gettext(String)
+#else
+#   define _(String) String
+#endif
+
 #include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
@@ -99,12 +106,10 @@ void dav_fuse5_loop(int device, size_t bufsize, time_t idle_time,
                     volatile int *keep_on_running) {
 
     buf_size = bufsize;
-    size_t headlen = sizeof(struct fuse_in_header) + sizeof(uint32_t)
-                     + sizeof(struct fuse_write_in);
-    buf = malloc(buf_size + headlen);
+    buf = malloc(buf_size);
     if (buf == NULL) {
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
-               "Could not allocate message buffer.");
+               _("can't allocate message buffer"));
         return;
     }
 
@@ -124,7 +129,7 @@ void dav_fuse5_loop(int device, size_t bufsize, time_t idle_time,
         int ret = select(device + 1, &fds, NULL, NULL, &tv);
 
         if (ret > 0) {
-            ssize_t bytes_read = read(device, buf, buf_size + headlen);
+            ssize_t bytes_read = read(device, buf, buf_size);
             if (bytes_read <= 0) {
                 if (bytes_read == 0 || errno == EINTR || errno == ENOENT) {
                     if (time(NULL) < (last_tidy_cache + idle_time)) {
@@ -525,7 +530,7 @@ static uint32_t fuse_open(void) {
         return sizeof(struct fuse_out_header);
     }
 
-    out->open_flags = in->flags & (O_ACCMODE | O_APPEND | O_FSYNC);
+    out->open_flags = in->flags & (O_ACCMODE | O_APPEND);
     out->fh = fd;
 
     DBG1("  fd %i", fd);
@@ -544,9 +549,9 @@ static uint32_t fuse_read(void) {
     DBG1("  pid %i", ih->pid);
     DBG2("  size %u, off %llu", in->size, in->offset);
 
-    if (in->size > buf_size) {
-        DBG_RET(EIO);
-        oh->error = -EIO;
+    if (in->size > (buf_size - sizeof(struct fuse_out_header))) {
+        DBG_RET(EINVAL);
+        oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
 
@@ -627,7 +632,6 @@ static uint32_t fuse_setattr(void) {
                             in->valid & FATTR_GID, in->attr.gid,
                             in->valid & FATTR_ATIME, in->attr.atime,
                             in->valid & FATTR_MTIME, in->attr.mtime,
-                            in->valid & FATTR_CTIME, in->attr.ctime,
                             in->valid & FATTR_SIZE, in->attr.size);
 
     DBG_RET(oh->error);
@@ -659,7 +663,7 @@ static uint32_t fuse_stat(void) {
     out->st.files = st.files;
     out->st.ffree = st.ffree;
     if (st.bsize > buf_size) {
-        out->st.bsize = buf_size & ~1023;
+        out->st.bsize = (buf_size - 4096) & ~1023;
     } else {
         out->st.bsize = st.bsize;
     }
@@ -683,9 +687,10 @@ static uint32_t fuse_write(void) {
     DBG2("  pid %i, flags 0%o", ih->pid, in->write_flags);
     DBG2("  size %u, off %llu", in->size, in->offset);
 
-    if (in->size > buf_size) {
-        DBG_RET(EIO);
-        oh->error = -EIO;
+    if (in->size > (buf_size - sizeof(struct fuse_in_header)
+                    - sizeof(struct fuse_write_in))) {
+        DBG_RET(EINVAL);
+        oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
 
