@@ -35,6 +35,13 @@
 #include <time.h>
 #include <unistd.h>
 
+#ifdef ENABLE_NLS
+#   include <libintl.h>
+#   define _(String) gettext(String)
+#else
+#   define _(String) String
+#endif
+
 #include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
@@ -114,12 +121,10 @@ void dav_fuse7_loop(int device, size_t bufsize, time_t idle_time,
                     volatile int *keep_on_running) {
 
     buf_size = bufsize;
-    size_t headlen = sizeof(struct fuse_in_header)
-                     + sizeof(struct fuse_write_in);
-    buf = malloc(buf_size + headlen);
+    buf = malloc(buf_size);
     if (buf == NULL) {
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
-               "Could not allocate message buffer.");
+               _("can't allocate message buffer"));
         return;
     }
 
@@ -139,9 +144,10 @@ void dav_fuse7_loop(int device, size_t bufsize, time_t idle_time,
         int ret = select(device + 1, &fds, NULL, NULL, &tv);
 
         if (ret > 0) {
-            ssize_t bytes_read = read(device, buf, buf_size + headlen);
+            ssize_t bytes_read = read(device, buf, buf_size);
             if (bytes_read <= 0) {
-                if (bytes_read == 0 || errno == EINTR || errno == ENOENT) {
+                if (bytes_read == 0 || errno == EINTR || errno == EAGAIN ||
+                        errno == ENOENT) {
                     if (time(NULL) < (last_tidy_cache + idle_time)) {
                         tv.tv_sec = last_tidy_cache + idle_time - time(NULL);
                     } else {
@@ -430,7 +436,8 @@ static uint32_t fuse_create(void) {
     }
 
     int fd = 0;
-    oh->error = dav_open(&fd, node, in->flags, ih->pid, 0, ih->uid);
+    oh->error = dav_open(&fd, node, in->flags & ~(O_EXCL | O_CREAT), ih->pid,
+                         0, ih->uid);
 
     if (oh->error != 0 || fd == 0) {
         if (created)
@@ -450,7 +457,7 @@ static uint32_t fuse_create(void) {
     out->entry.attr_valid_nsec = 0;
     set_attr(&out->entry.attr, node);
 
-    out->open.open_flags = in->flags & (O_ACCMODE | O_APPEND | O_FSYNC);
+    out->open.open_flags = in->flags & (O_ACCMODE | O_APPEND);
     out->open.fh = fd;
     out->open.padding = 0;
 
@@ -508,7 +515,8 @@ static uint32_t fuse_init(void) {
     out->unused[0] = 0;
     out->unused[1] = 0;
     out->unused[2] = 0;
-    out->max_write = buf_size;
+    out->max_write = buf_size - sizeof(struct fuse_in_header)
+                     - sizeof(struct fuse_write_in);
 
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_init_out);
 }
@@ -652,7 +660,7 @@ static uint32_t fuse_open(void) {
         return sizeof(struct fuse_out_header);
     }
 
-    out->open_flags = in->flags & (O_ACCMODE | O_APPEND | O_FSYNC);
+    out->open_flags = in->flags & (O_ACCMODE | O_APPEND);
     out->fh = fd;
     out->padding = 0;
 
@@ -672,9 +680,9 @@ static uint32_t fuse_read(void) {
     DBG1("  pid %i", ih->pid);
     DBG2("  size %u, off %llu", in->size, in->offset);
 
-    if (in->size > buf_size) {
-        DBG_RET(EIO);
-        oh->error = -EIO;
+    if (in->size > (buf_size - sizeof(struct fuse_out_header))) {
+        DBG_RET(EINVAL);
+        oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
 
@@ -755,7 +763,6 @@ static uint32_t fuse_setattr(void) {
                             in->valid & FATTR_GID, in->gid,
                             in->valid & FATTR_ATIME, in->atime,
                             in->valid & FATTR_MTIME, in->mtime,
-                            0, 0,
                             in->valid & FATTR_SIZE, in->size);
 
     DBG_RET(oh->error);
@@ -787,7 +794,7 @@ static uint32_t fuse_stat(void) {
     out->st.files = st.files;
     out->st.ffree = st.ffree;
     if (st.bsize > buf_size) {
-        out->st.bsize = buf_size & ~1023;
+        out->st.bsize = (buf_size - 4096) & ~1023;
     } else {
         out->st.bsize = st.bsize;
     }
@@ -816,9 +823,10 @@ static uint32_t fuse_write(void) {
     DBG2("  pid %i, flags 0%o", ih->pid, in->write_flags);
     DBG2("  size %u, off %llu", in->size, in->offset);
 
-    if (in->size > buf_size) {
-        DBG_RET(EIO);
-        oh->error = -EIO;
+    if (in->size > (buf_size - sizeof(struct fuse_in_header)
+                    - sizeof(struct fuse_write_in))) {
+        DBG_RET(EINVAL);
+        oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
 
