@@ -41,10 +41,6 @@
    command line and even fstab will be silently ignored. */
 #define DAV_MOPTS (MS_MGC_VAL | MS_NOSUID | MS_NODEV)
 
-/* Ordinary users must be member of this group to be able to mount.
-   May be overridden by system config file. */
-#define DAV_GROUP "users"
-
 /* XML namespace for the cache index file. */
 #define DAV_XML_NS "http://dav.sf.net/"
 
@@ -59,27 +55,10 @@
    use _PATH_MOUNTED. */
 #define DAV_MOUNTS "/proc/mounts"
 
-/* The directory where the cache files will be stored, when mounted by root.
-   May be overridden by system config file. */
-#define DAV_SYS_CACHE "/var/cache/" PACKAGE_TARNAME
-
-/* The directory where the pid-files will be placed. */
-#define DAV_SYS_RUN "/var/run/" PROGRAM_NAME
-
-/* Directory holding user config files and cache directory. */
-#define DAV_USER_DIR "~/." PACKAGE_TARNAME
-
-/* User configuration file; relative to DAV_USER_DIR.
-   May be overridden by command line or fstab. */
-#define DAV_USER_CONF PACKAGE_TARNAME ".conf"
-
-/* User file holding credentials; relative to DAV_USER_DIR. */
-#define DAV_USER_SECRETS "secrets"
-
 /* The directory where the cache files will be stored, when mounted by
    a non root users; relative to DAV_USER_DIR.
    May be overridden by user config file. */
-#define DAV_USER_CACHE "cache"
+#define DAV_CACHE "cache"
 
 /* The name of index files. */
 #define DAV_INDEX "index"
@@ -114,7 +93,7 @@
 /* How long in seconds a cached directory is valid. After this time
    a new PROPFIND request for this directory must be performed.
    May be overridden by system config file and user config file. */
-#define DAV_EXPIRE 300
+#define DAV_DIR_REFRESH 60
 
 /* Wait at least that many seconds from last file access until a new
    GET If-Modified request is send to the server. If set to 0 a request
@@ -122,7 +101,12 @@
    open and close calls in short sequence that cause - mostly - unnecessary
    traffic.
    May be overridden by system config file and user config file. */
-#define DAV_GET_WAIT 1
+#define DAV_FILE_REFRESH 1
+
+/* Use PROPFIND to get the Last-Modified time of all files in a directory
+   instead of GET If-Modified_Since for single files.
+   May be overridden by system config file and user config file. */
+#define DAV_GUI_OPTIMIZE 0
 
 /* Default retry time after a PROPFIND request failed. When the request fails
    again, the retry time will subsequently be increased up to DAV_MAX_RETRY.
@@ -166,6 +150,10 @@
    May be overridden by command line or fstab. */
 #define DAV_LOCKS 1
 
+/* Send expect 100-continue header in PUT requests.
+   May be overridden by system config file and user config file. */
+#define DAV_EXPECT100 1
+
 /* Timeout in seconds used when reading from a socket.
    May be overridden by system config file and user config file. */
 #define DAV_READ_TIMEOUT 30
