@@ -34,11 +34,15 @@
    Some data will be copied into global or local variables to be available in
    daemon mode. The rest will be freed when forking into daemon mode. */
 typedef struct {
-    gid_t dav_group;          /* System config file */
+    char *dav_user;           /* System config file */
+    char *dav_group;          /* System config file */
+    char *ignore_home;        /* System config file */
     char *conf;               /* Command line */
     /* Mount options */
     int user;                 /* Command line */
+    int netdev;               /* Command line */
     unsigned long int mopts;  /* Command line */
+    char *add_mopts;
     char *kernel_fs;          /* User config file, system config file */
     size_t buf_size;          /* User config file, system config file */
     /* File mode */
@@ -53,9 +57,12 @@ typedef struct {
     char *host;               /* Command line */
     int port;                 /* Command line */
     char *path;               /* Command line */
+    char *servercert;         /* User config file, system config file */
     char *secrets;            /* User config file */
     char *username;           /* User secrets file, system secrets file */
     char *password;           /* User secrets file, system secrets file */
+    char *clicert;            /* User config file, system config file */
+    char *clicert_pw;         /* User secrets file, system secrets file */
     char *p_host;             /* User config file, sys conf f., environment */
     int p_port;               /* User config file, sys conf f., environment */
     char *p_user;             /* User secrets file, system secrets file */
@@ -64,7 +71,9 @@ typedef struct {
     int askauth;              /* User config file, sys conf f., command line */
     int displayname;          /* User config file, system config file */
     int locks;                /* User config file, sys conf f., command line */
+    int expect100;            /* User config file, system config file */
     time_t read_timeout;      /* User config file, system config file */
+    char * s_charset;         /* User config file, system config file */
     /* Cache */
     char *sys_cache;          /* System config file */
     char *cache_dir;          /* User config file */
@@ -72,8 +81,9 @@ typedef struct {
     size_t cache_size;        /* User config file, system config file */
     size_t table_size;        /* User config file, system config file */
     time_t idle_time;         /* User config file, system config file */
-    time_t expire;            /* User config file, system config file */
-    time_t get_wait;          /* User config file, system config file */
+    time_t dir_refresh;       /* User config file, system config file */
+    time_t file_refresh;      /* User config file, system config file */
+    int gui_optimize;         /* User config file, system config file */
     time_t retry;             /* User config file, system config file */
     time_t max_retry;         /* User config file, system config file */
     time_t lock_timeout;      /* User config file, system config file */
