@@ -27,6 +27,8 @@
 #include <errno.h>
 #include <error.h>
 #include <fcntl.h>
+#include <iconv.h>
+#include <langinfo.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,6 +37,15 @@
 #include <time.h>
 #include <unistd.h>
 
+#include <sys/stat.h>
+
+#ifdef ENABLE_NLS
+#   include <libintl.h>
+#   define _(String) gettext(String)
+#else
+#   define _(String) String
+#endif
+
 #include <ne_alloc.h>
 #include <ne_auth.h>
 #include <ne_basic.h>
@@ -47,8 +58,7 @@
 #include <ne_string.h>
 #include <ne_uri.h>
 #include <ne_utils.h>
-
-#include <sys/stat.h>
+#include <ne_xml.h>
 
 #include "dav_debug.h"
 #include "defaults.h"
@@ -63,6 +73,7 @@
 typedef struct {
     const char *path;           /* The *not* url-encoded path. */
     dav_props *results;         /* Start of the linked list of dav_props. */
+    const ne_xml_parser *parser; /* the parser, to get the encoding from it. */
 } propfind_context;
 
 typedef struct {
@@ -122,18 +133,40 @@ static char *p_password;
 /* Whether to use the displayname property for file and directory names. */
 int use_displayname;
 
+/* Whether to send expect 100-continue header in PUT requests. */
+int use_expect100;
+
 /* The url for this session. Contains anything needed except path.
    Will be initialized by dav_init_webdav() and used by various functions.
    These will add path, use suri in calls to neon functions and finally
    free path. path must allways be initialized before use, as it may
    point to anything. */
 static ne_uri *suri;
-   
+
+/* Will be set to 1 when dav_init_connection() succeeded. */
+static int initialized;
+
 /* Whether a terminal is available to communicate with the user.
    Should be reset with set_no_terminal() when forking into daemon mode.
    Needed by  ssl_verify() which may be called at any time. */
 static int have_terminal = 1;
 
+/* Handle to convert character set from utf-8 encoded displaynames to LC_CTYPE.
+   If NULL no conversion is done. */
+static iconv_t from_utf_8;
+
+/* Handle to convert character set from utf-8 encoded displaynames to LC_CTYPE.
+   If NULL no conversion is done. */
+static iconv_t from_utf_16;
+
+/* Handle to convert character set from path names to LC_CTYPE. If NULL
+   no conversion is done. */
+static iconv_t from_s_charset;
+
+/* Handle to convert from LC_CTYPE to character set of path names. If NULL
+   no conversion is done. */
+static iconv_t to_s_charset;
+
 
 /* Private function prototypes and inline functions */
 /*==================================================*/
@@ -182,19 +215,40 @@ static int ssl_verify(void *userdata, int failures,
 /*==================*/
 
 void dav_init_webdav(const char *scheme, const char *host, int port,
-                     time_t read_timeout, int use_locks, time_t l_timeout,
+                     time_t read_timeout, int use_locks,
+                     time_t l_timeout, const char *servercert,
+                     const char *clicert, const char *clicert_pw, int askauth,
                      const char *user, const char *passwd, int useproxy,
                      const char *p_host, int p_port, const char *p_user,
                      const char *p_passwd, const char *lock_owner,
-                     int displayname) {
+                     int displayname, int expect100, const char *s_charset) {
+
+    char *lc_charset = nl_langinfo(CODESET);
+    if (lc_charset != NULL && strcasecmp(lc_charset, "UTF-8") != 0)
+        from_utf_8 = iconv_open(lc_charset, "UTF-8");
+    if (lc_charset != NULL && strcasecmp(lc_charset, "UTF-16") != 0)
+        from_utf_16 = iconv_open(lc_charset, "UTF-16");
+
+    if (lc_charset != NULL && s_charset != NULL
+            && (strcasecmp(s_charset, lc_charset) != 0)) {
+        if (strcasecmp(s_charset, "UTF-8") == 0) {
+            from_s_charset = from_utf_8;
+        } else if (strcasecmp(s_charset, "UTF-16") == 0) {
+            from_s_charset = from_utf_16;
+        } else {
+            from_s_charset = iconv_open(lc_charset, s_charset);
+        }
+        to_s_charset = iconv_open(s_charset, lc_charset);
+    }
 
     if (ne_sock_init() != 0)
-        error(EXIT_FAILURE, 0, "Socket library initalization failed");
+        error(EXIT_FAILURE, errno, _("socket library initalization failed"));
 
     session = ne_session_create(scheme, host, port);
 
 #if NE_VERSION_MINOR == 24
-    ne_set_expect100(session, 1);
+    if (expect100)
+        ne_set_expect100(session, 1);
 #endif
 
     ne_set_read_timeout(session, read_timeout);
@@ -202,27 +256,13 @@ void dav_init_webdav(const char *scheme, const char *host, int port,
     char *useragent = ne_concat(PACKAGE_TARNAME, "/", PACKAGE_VERSION, NULL);
     ne_set_useragent(session, useragent);
     free(useragent);
-    
-    if (use_locks) {
-        locks = ne_lockstore_create();
-        ne_lockstore_register(locks, session);
-        if (lock_owner == NULL) {
-            if (user == NULL) {
-                owner = ne_concat("unknown/", PACKAGE_STRING, NULL);
-            } else {
-                owner = ne_concat(user, "/", PACKAGE_STRING, NULL);
-            }
-        } else {
-            owner = ne_strdup(lock_owner);
-        }
-    }
 
     if (user != NULL)
         username = ne_strdup(user);
     if (passwd != NULL)
         password = ne_strdup(passwd);
     ne_set_server_auth(session, auth, "server");
-    
+
     if (useproxy && p_host != NULL) {
         ne_session_proxy(session, p_host, p_port);
         if (p_user != NULL)
@@ -231,27 +271,130 @@ void dav_init_webdav(const char *scheme, const char *host, int port,
             p_password = ne_strdup(p_passwd);
         ne_set_proxy_auth(session, auth, "proxy");
     }
-    
+
     if (strcmp(scheme, "https") == 0) {
 #if NE_VERSION_MINOR == 24
         if (!ne_supports_ssl())
 #else
         if (!ne_has_support(NE_FEATURE_SSL))
 #endif
-            error(EXIT_FAILURE, 0, "Neon library does not support TLS/SSL.");
+            error(EXIT_FAILURE, 0, _("neon library does not support TLS/SSL"));
         ne_ssl_set_verify(session, ssl_verify, NULL);
         ne_ssl_trust_default_ca(session);
+
+        if (servercert != NULL) {
+            ne_ssl_certificate *server_cert = ne_ssl_cert_read(servercert);
+            if (server_cert == NULL)
+                error(EXIT_FAILURE, 0, _("can't read server certificate %s"),
+                      servercert);
+            ne_ssl_trust_cert(session, server_cert);
+            ne_ssl_cert_free(server_cert);
+        }
+
+        if (clicert != NULL) {
+            uid_t orig = geteuid();
+            seteuid(0);
+            ne_ssl_client_cert *client_cert = ne_ssl_clicert_read(clicert);
+            seteuid(orig);
+            if (client_cert == NULL)
+                error(EXIT_FAILURE, 0, _("can't read client certificate %s"),
+                      clicert);
+            if (client_cert != NULL && ne_ssl_clicert_encrypted(client_cert)) {
+                char *pw = NULL;
+                if (clicert_pw == NULL && askauth) {
+                    printf(_("Please enter the password to decrypt client\n"
+                             "certificate %s.\n"), clicert);
+                    pw = getpass(_("Passord: "));
+                } else {
+                    pw = ne_strdup(clicert_pw);
+                }
+                int ret = 1;
+                if (pw != NULL) {
+                    ret = ne_ssl_clicert_decrypt(client_cert, pw);
+                    memset(pw, '\0', strlen(pw));
+                    free(pw);
+                }
+                if (ret)
+                    error(EXIT_FAILURE, 0,
+                          _("can't decrypt client certificate %s"), clicert);
+            }
+            ne_ssl_set_clicert(session, client_cert);
+            ne_ssl_clicert_free(client_cert);
+        }
     }
 
     suri = (ne_uri *) ne_calloc(sizeof(ne_uri));
     ne_fill_server_uri(session, suri);
-    
+
     use_displayname = displayname;
+    use_expect100 = expect100;
+
+    if (use_locks) {
+        locks = ne_lockstore_create();
+        ne_lockstore_register(locks, session);
+        if (lock_owner == NULL) {
+            if (user == NULL) {
+                owner = ne_strdup(PACKAGE_STRING);
+            } else {
+                owner = ne_strdup(user);
+            }
+        } else {
+            owner = ne_strdup(lock_owner);
+        }
+        lock_timeout = l_timeout;
+    }
+}
+
+
+int dav_init_connection(const char *path) {
+
+    suri->path = ne_path_escape(path);
+    if (suri->path == NULL)
+        abort();
+
+    ne_server_capabilities caps = {0, 0, 0};
+    int ret = ne_options(session, path, &caps);
+
+    if (ret == 0) {
+        initialized = 1;
+        if (!caps.dav_class1) {
+            if (have_terminal) {
+                error(EXIT_FAILURE, 0,
+                      _("mounting failed; the server does not support WebDAV"));
+            } else {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
+                       _("mounting failed; the server does not support WebDAV"));
+                ret = EINVAL;
+            }
+        }
+        if (!caps.dav_class2) {
+            if (have_terminal)
+                error(0, 0, _("warning: the server does not support locks"));
+            if (locks != NULL)
+                ne_lockstore_destroy(locks);
+            locks = NULL;
+        }
+    } else {
+        ret = get_error(ret, "OPTIONS");
+    }
+
+    free(suri->path);
+    return ret;
 }
 
 
 void dav_close_webdav(void) {
 
+    if (from_utf_8 > 0 )
+        iconv_close(from_utf_8);
+    if (from_utf_16 > 0 )
+        iconv_close(from_utf_16);
+    if (from_s_charset > 0 && from_s_charset != from_utf_8
+            && from_s_charset != from_utf_16)
+        iconv_close(from_s_charset);
+    if (to_s_charset > 0)
+        iconv_close(to_s_charset);
+
     if(session == NULL)
         return;
 
@@ -272,6 +415,29 @@ void dav_close_webdav(void) {
 }
 
 
+char *dav_cconvert(const char *name) {
+
+    char *converted = ne_strdup(name);;
+    if (to_s_charset > 0) {
+        size_t insize = strlen(converted);
+        char *in = converted;
+        size_t outsize = MB_LEN_MAX * insize;
+        char *buf = calloc(outsize + 1, 1);
+        if (buf == NULL)
+            abort();
+        char *out = buf;
+        iconv(to_s_charset, NULL, NULL, &out, &outsize);
+        if (iconv(to_s_charset, &in, &insize, &out, &outsize) >= 0
+                && insize == 0) {
+            free(converted);
+            converted = ne_strdup(buf);
+        }
+        free(buf);
+    }
+    return converted;
+}
+
+
 int dav_delete(const char *path, time_t *expire) {
 
     suri->path = ne_path_escape(path);
@@ -288,8 +454,10 @@ int dav_delete(const char *path, time_t *expire) {
 
     if (ret == 0 && locks != NULL && *expire != 0) {
         struct ne_lock *lock = ne_lockstore_findbyuri(locks, suri);
-        if (lock != NULL)
+        if (lock != NULL) {
             ne_lockstore_remove(locks, lock);
+        		ne_lock_destroy(lock);
+        }
     }
 
     free(suri->path);
@@ -299,12 +467,19 @@ int dav_delete(const char *path, time_t *expire) {
 
 int dav_delete_dir(const char *path) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
     suri->path = ne_path_escape(path);
     if (suri->path == NULL)
         abort();
 
     DBG1("  DELETE %s", suri->path);
-    int ret = ne_delete(session, suri->path);
+    ret = ne_delete(session, suri->path);
     ret = get_error(ret, "DELETE");
 
     free(suri->path);
@@ -326,6 +501,13 @@ void dav_delete_props(dav_props *props) {
 
 int dav_get_collection(const char *path, dav_props **props) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
     propfind_context ctx;
     ctx.path = path;
     ctx.results = NULL;
@@ -335,8 +517,9 @@ int dav_get_collection(const char *path, dav_props **props) {
         abort();
     ne_propfind_handler *pfh = ne_propfind_create(session, suri->path,
                                                   NE_DEPTH_ONE);
+    ctx.parser = ne_propfind_get_parser(pfh);
     DBG1("  PROPFIND %s", suri->path);
-    int ret = ne_propfind_named(pfh, prop_names, prop_result, &ctx);
+    ret = ne_propfind_named(pfh, prop_names, prop_result, &ctx);
     ret = get_error(ret, "PROPFIND");
     ne_propfind_destroy(pfh);
     free(suri->path);
@@ -354,8 +537,21 @@ int dav_get_collection(const char *path, dav_props **props) {
 }
 
 
+const char *dav_get_webdav_error() {
+
+    return ne_get_error(session);
+}
+
+
 int dav_get_file(const char *path, const char *cache_path, off_t *size,
-                 char **etag, time_t *mtime) {
+                 char **etag, time_t *mtime, int *modified) {
+
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
 
     get_context ctx;
     ctx.error = 0;
@@ -391,7 +587,7 @@ int dav_get_file(const char *path, const char *cache_path, off_t *size,
     ne_add_response_body_reader(req, ne_accept_2xx, block_writer, &ctx);
 
     DBG1("  GET %s", suri->path);
-    int ret = ne_request_dispatch(req);
+    ret = ne_request_dispatch(req);
     ret = get_error(ret, "GET");
     if (ctx.error)
         ret = ctx.error;
@@ -401,6 +597,8 @@ int dav_get_file(const char *path, const char *cache_path, off_t *size,
 #if NE_VERSION_MINOR == 24
 
     if (ret == 0 && status->code == 200) {
+        if (modified != NULL)
+            *modified = 1;
         *size = total;
         if (mtime != NULL && r_mod_time != NULL)
             *mtime = ne_httpdate_parse(r_mod_time);
@@ -425,6 +623,8 @@ int dav_get_file(const char *path, const char *cache_path, off_t *size,
 #else     /* NE_VERSION_MINOR */
 
     if (ret == 0 && status->code == 200) {
+        if (modified != NULL)
+            *modified = 1;
         const char *value = ne_get_response_header(req, "Content-Length");
         if (value != NULL)
 #if _FILE_OFFSET_BITS == 64
@@ -463,6 +663,13 @@ int dav_get_file(const char *path, const char *cache_path, off_t *size,
 
 int dav_head(const char *path, char **etag, time_t *mtime) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
     suri->path = ne_path_escape(path);
     if (suri->path == NULL)
         abort();
@@ -479,9 +686,8 @@ int dav_head(const char *path, char **etag, time_t *mtime) {
 #endif    /* NE_VERSION_MINOR */
 
     DBG1("  HEAD %s", suri->path);
-    int ret = ne_request_dispatch(req);
+    ret = ne_request_dispatch(req);
     ret = get_error(ret, "HEAD");
-    ne_request_destroy(req);
 
 #if NE_VERSION_MINOR == 24
 
@@ -521,6 +727,7 @@ int dav_head(const char *path, char **etag, time_t *mtime) {
 
 #endif    /* NE_VERSION_MINOR */
 
+    ne_request_destroy(req);
     free(suri->path);
     return ret;
 }
@@ -528,6 +735,13 @@ int dav_head(const char *path, char **etag, time_t *mtime) {
 
 int dav_lock(const char *path, time_t *expire, int *exists) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
     if (locks == NULL) {
         *expire = 0;
         return 0;
@@ -555,7 +769,7 @@ int dav_lock(const char *path, time_t *expire, int *exists) {
     lock->timeout = lock_timeout;
 
     DBG1("  LOCK %s", lock->uri.path);
-    int ret = ne_lock(session, lock);
+    ret = ne_lock(session, lock);
     ret = get_error(ret, "LOCK");
 
     if (ret == 0) {
@@ -583,6 +797,9 @@ int dav_lock(const char *path, time_t *expire, int *exists) {
 
 void dav_lock_refresh(const char *path, time_t *expire) {
 
+    if (!initialized && dav_init_connection(path) != 0)
+        return;
+
     if (locks == NULL) {
         *expire = 0;
         return;
@@ -605,11 +822,18 @@ void dav_lock_refresh(const char *path, time_t *expire) {
 
 int dav_make_collection(const char *path) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
     suri->path = ne_path_escape(path);
     if (suri->path == NULL)
         abort();
     DBG1("  MKCOL %s", suri->path);
-    int ret = ne_mkcol(session, suri->path);
+    ret = ne_mkcol(session, suri->path);
     ret = get_error(ret, "MKCOL");
     free(suri->path);
     return ret;
@@ -618,6 +842,13 @@ int dav_make_collection(const char *path) {
 
 int dav_move(const char *src, const char *dst) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(src);
+        if (ret != 0)
+            return ret;
+    }
+
     suri->path = ne_path_escape(src);
     char *dst_path = ne_path_escape(dst);
     if (suri->path == NULL || dst_path == NULL)
@@ -625,7 +856,7 @@ int dav_move(const char *src, const char *dst) {
 
     DBG1("  MOVE %s", suri->path);
     DBG1("       %s", dst_path);
-    int ret = ne_move(session, 1, suri->path, dst_path); 
+    ret = ne_move(session, 1, suri->path, dst_path); 
     ret = get_error(ret, "MOVE");
 
     free(suri->path);
@@ -635,28 +866,37 @@ int dav_move(const char *src, const char *dst) {
 
 
 int dav_put(const char *path, const char *cache_path, int *exists,
-            time_t *expire, char **etag, time_t *mtime) {
+            time_t *expire, char **etag, time_t *mtime, int execute) {
 
     int ret = 0;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
 
     if (*exists == 0 || (etag != NULL && *etag != NULL)
             || (mtime != NULL && *mtime != 0)) {
         char *r_etag = NULL;
         time_t r_mtime = 0;
         ret = dav_head(path, &r_etag, &r_mtime);
-        if (ret == 0 && *exists == 0) {
-            ret = EEXIST;
-        } else if (ret == ENOENT && *exists == 0) {
-            ret = 0;
+        if (ret == 0) {
+            if (*exists == 0) {
+                ret = EEXIST;
+            } else if (etag != NULL && *etag != NULL && r_etag != NULL) {
+                if (strcmp(*etag, r_etag) != 0)
+                    ret = EINVAL;
+            } else if (mtime != NULL && *mtime != 0 && r_mtime != 0) {
+                if (*mtime < r_mtime)
+                    ret = EINVAL;
+            }
         } else if (ret == ENOENT) {
-            if (dav_unlock(path, expire) == 0)
+            if (*exists == 0) {
                 ret = 0;
-        } else if (ret == 0 && etag != NULL && *etag != NULL && r_etag != NULL
-                   && strcmp(*etag, r_etag) != 0) {
-            ret = EINVAL;
-        } else if (ret == 0 && mtime != NULL && *mtime != 0 && r_mtime != 0
-                   && *mtime < r_mtime) {
-            ret = EINVAL;
+            } else {
+                ret = dav_unlock(path, expire);
+            }
         }
         if (r_etag != NULL)
             free(r_etag);
@@ -678,6 +918,8 @@ int dav_put(const char *path, const char *cache_path, int *exists,
 
     if (ret == 0) {
         *exists = 1;
+        if (execute == 1)
+            dav_set_execute(path, execute);
         dav_head(path, etag, mtime);
     }
 
@@ -687,6 +929,13 @@ int dav_put(const char *path, const char *cache_path, int *exists,
 
 int dav_set_execute(const char *path, int set) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
     ne_proppatch_operation op[2];
     op[0].name = &prop_names[EXECUTE];
     op[0].type = ne_propset;
@@ -701,7 +950,7 @@ int dav_set_execute(const char *path, int set) {
     if (suri->path == NULL)
         abort();
     DBG1("  PROPPATCH %s", suri->path);
-    int ret = ne_proppatch(session, suri->path, &op[0]);
+    ret = ne_proppatch(session, suri->path, &op[0]);
     ret = get_error(ret, "PROPPATCH");
     free(suri->path);
 
@@ -716,6 +965,13 @@ void dav_set_no_terminal(void) {
 
 int dav_unlock(const char *path, time_t *expire) {
 
+    int ret;
+    if (!initialized) {
+        ret = dav_init_connection(path);
+        if (ret != 0)
+            return ret;
+    }
+
     if (locks == NULL) {
         *expire = 0;
         return 0;
@@ -733,7 +989,7 @@ int dav_unlock(const char *path, time_t *expire) {
     }
 
     DBG1("  UNLOCK %s", lock->uri.path);
-    int ret = ne_unlock(session, lock);
+    ret = ne_unlock(session, lock);
     ret = get_error(ret, "UNLOCK");
     if (ret == 0 || ret == ENOENT || ret == EINVAL) {
         ne_lockstore_remove(locks, lock);
@@ -1001,10 +1257,11 @@ static int put_file(const char *spath, int fd, time_t *expire,
         return EIO;
 
     ne_request *req = ne_request_create(session, "PUT", spath);
+    if (use_expect100)
 #if NE_VERSION_MINOR == 25
-    ne_set_request_expect100(req, 1);
+        ne_set_request_expect100(req, 1);
 #else
-    ne_set_request_flag(req, NE_REQFLAG_EXPECT100, 1);
+        ne_set_request_flag(req, NE_REQFLAG_EXPECT100, 1);
 #endif
     ne_lock_using_resource(req, spath, 0);
 #if _FILE_OFFSET_BITS == 64
@@ -1020,10 +1277,11 @@ static int put_file(const char *spath, int fd, time_t *expire,
     if (ret == EACCES && lock_discover(spath, expire) == 0) {
         ne_request_destroy(req);
         ne_request *req = ne_request_create(session, "PUT", spath);
+        if (use_expect100)
 #if NE_VERSION_MINOR == 25
-        ne_set_request_expect100(req, 1);
+            ne_set_request_expect100(req, 1);
 #else
-        ne_set_request_flag(req, NE_REQFLAG_EXPECT100, 1);
+            ne_set_request_flag(req, NE_REQFLAG_EXPECT100, 1);
 #endif
         ne_lock_using_resource(req, spath, 0);
 #if _FILE_OFFSET_BITS == 64
@@ -1070,7 +1328,7 @@ static int auth(void *userdata, const char *realm, int attempt, char *user,
 
     DBG1("  Neon wants creditentials for %s.", realm);
     if (attempt != 0) {
-        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "Authentication failure:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), _("authentication failure:"));
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "  %s", realm);
         return attempt;
     }
@@ -1116,7 +1374,7 @@ static int block_writer(void *userdata, const char *block, size_t length) {
     if (ctx->fd == 0)
         ctx->fd = open(ctx->file, O_WRONLY | O_CREAT | O_TRUNC);
     if (ctx->fd <= 0) {
-        ne_set_error(session, "0 Could not open cache file.");
+        ne_set_error(session, _("%i can't open cache file"), 0);
         ctx->error = EIO;
     }
 
@@ -1124,7 +1382,7 @@ static int block_writer(void *userdata, const char *block, size_t length) {
         ssize_t ret = write(ctx->fd, block, length);
         if (ret < 0) {
             ctx->error = EIO;
-            ne_set_error(session, "0 Error writing to cache file.");
+            ne_set_error(session, _("%i error writing to cache file"), 0);
         } else {
             length -= ret;
             block += ret;
@@ -1265,8 +1523,15 @@ static void prop_result(void *userdata, const ne_uri *uri,
         } else {
             data = NULL;
         }
+        iconv_t converter = 0;
         if (data != NULL) {
             result->name = ne_strdup(data);
+            const char *encoding = ne_xml_doc_encoding(ctx->parser);
+            if ( !encoding || strcasecmp(encoding, "UTF-8") == 0) {
+                converter = from_utf_8;
+            } else if (strcasecmp(encoding, "UTF-16") == 0) {
+                converter = from_utf_16;
+            }
         } else {
             if (strlen(result->path)
                     < (strlen(ctx->path) + result->is_dir + 1)) {
@@ -1276,15 +1541,41 @@ static void prop_result(void *userdata, const ne_uri *uri,
             result->name = ne_strndup(result->path + strlen(ctx->path),
                                       strlen(result->path) - strlen(ctx->path)
                                       - result->is_dir);
+            converter = from_s_charset;
+        }
+        if (converter > 0) {
+            size_t insize = strlen(result->name);
+            char *in = result->name;
+            size_t outsize = MB_LEN_MAX * insize;
+            char *buf = calloc(outsize + 1, 1);
+            if (buf == NULL)
+                abort();
+            char *out = buf;
+            iconv(converter, NULL, NULL, &out, &outsize);
+            if (iconv(converter, &in, &insize, &out, &outsize) >= 0
+                    && insize == 0) {
+                free(result->name);
+                result->name = ne_strdup(buf);
+            }
+            free(buf);
         }
     }
 
+    char *end = result->name + strlen(result->name) -1;
     char *slash = strchr(result->name, '/');
     while (slash != NULL) {
+        char *name;
         *slash = '\0';
-        char *name = ne_concat(result->name, "-slash-", slash + 1, NULL);
+        if (slash == result->name) {
+            name = ne_concat("slash-", slash + 1, NULL);
+        } else if (slash == end) {
+            name = ne_concat(result->name, "-slash", NULL);
+        } else {
+            name = ne_concat(result->name, "-slash-", slash + 1, NULL);
+        }
         free(result->name);
         result->name = name;
+        end = result->name + strlen(result->name) -1;
         slash = strchr(result->name, '/');
     }
 
@@ -1381,54 +1672,72 @@ static int ssl_verify(void *userdata, int failures,
     int ret = 0;
     if (issuer == NULL || subject == NULL
             || ne_ssl_cert_digest(cert, digest) != 0) {
+        if (have_terminal) {
+            error(0, 0, _("error processing server certificate"));
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
+                   _("error processing server certificate"));
+        }
         ret = -1;
     }
 
-    char *error_msg;
-    if (ret == -1) {
-        error_msg = "Error processing certificate.";
-    } else if (failures & NE_SSL_NOTYETVALID) {
-        error_msg = "The server certificate is not yet valid.";
-    } else if (failures & NE_SSL_EXPIRED) {
-        error_msg = "The server certificate has expired.";
-    } else if (failures & NE_SSL_IDMISMATCH) {
-        error_msg = "The server certificate does not match the server name.";
-    } else if (failures & NE_SSL_UNTRUSTED) {
-        error_msg = "The server certificate is not trusted.";
-    } else {
-        error_msg = "Unrecognized certificate error.";
-    }
-
     if (have_terminal) {
-        printf("%s\n", error_msg);
-        printf("  Server identity: %s\n", ne_ssl_cert_identity(cert));
-        printf("  Issuer:  %s\n", issuer);
-        printf("  Subject: %s\n", subject);
-        printf("  Fingerprint: %s\n", digest);
+        if (failures & NE_SSL_NOTYETVALID)
+            error(0, 0, _("the server certificate is not yet valid"));
+        if (failures & NE_SSL_EXPIRED)
+            error(0, 0, _("the server certificate has expired"));
+        if (failures & NE_SSL_IDMISMATCH)
+            error(0, 0, _("the server certificate does not match the server name"));
+        if (failures & NE_SSL_UNTRUSTED)
+            error(0, 0, _("the server certificate is not trusted"));
+        if (failures & ~NE_SSL_FAILMASK)
+            error(0, 0, _("unknown certificate error"));
+        printf(_("  issuer:      %s"), issuer);
+        printf("\n");
+        printf(_("  subject:     %s"), subject);
+        printf("\n");
+        printf(_("  identity:    %s"), ne_ssl_cert_identity(cert));
+        printf("\n");
+        printf(_("  fingerprint: %s"), digest);
+        printf("\n");
         if (ret == 0) {
-            printf("\nYou only should accept this certificate, if you can\n");
-            printf("verify the fingerprint! The server might be faked\n");
-            printf("or there might be a man-in-the-middle-attack.\n");
-            printf("Accept certificate for this session? [y,N] ");
+            printf(_("You only should accept this certificate, if you can\n"
+                     "verify the fingerprint! The server might be faked\n"
+                     "or there might be a man-in-the-middle-attack.\n"));
+            printf(_("Accept certificate for this session? [y,N] "));
             char *s = NULL;
             size_t n = 0;
             ssize_t len = 0;
             len = getline(&s, &n, stdin);
             if (len < 0)
                 abort();
-            if (!(*s == 'y' || *s == 'Y'))
+            if (rpmatch(s) < 1)
                 ret = -1;
             free(s);
         }
-    }
+    } 
 
-    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), error_msg);
-    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "  identity:  %s",
+    if (failures & NE_SSL_NOTYETVALID)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
+               _("the server certificate is not yet valid"));
+    if (failures & NE_SSL_EXPIRED)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
+               _("the server certificate has expired"));
+    if (failures & NE_SSL_IDMISMATCH)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
+               _("the server certificate does not match the server name"));
+    if (failures & NE_SSL_UNTRUSTED)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
+               _("the server certificate is not trusted"));
+    if (failures & ~NE_SSL_FAILMASK)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
+               _("unknown certificate error"));
+    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), _("  issuer: %s"), issuer);
+    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), _("  subject: %s"), subject);
+    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), _("  identity: %s"),
                        ne_ssl_cert_identity(cert));
-    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "  issuer:   %s", issuer);
-    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "  subject:  %s", subject);
     if (ret == 0) {
-        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "  Accepted by user.");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), _("  accepted by user"));
     }
 
     free(issuer);
