@@ -52,16 +52,28 @@ struct dav_props {
 /*=====================*/
 
 /* Creates and initializes a neon_session, using configuration information
-   given as parameters.
+   given as parameters, and checks the WebDAV class of the server.
+   If the server does not support class 2, locking is disabled.
    It must only be initialized once, as it depends on global variables.
    If an error occurs, the program is terminated.
-   paramters: if not self explaining, please see mount_davfs.c, struct args. */
+   paramters: if not self explaining, please see mount_davfs.h, struct args. */
 void dav_init_webdav(const char *scheme, const char *host, int port,
-                     time_t read_timeout, int use_locks, time_t l_timeout,
+                     time_t read_timeout, int use_locks,
+                     time_t l_timeout, const char *servercert,
+                     const char *clicert, const char *clicert_pw, int askauth,
                      const char *user, const char *passwd, int useproxy,
                      const char *p_host, int p_port, const char *p_user,
                      const char *p_passwd, const char *lock_owner,
-                     int displayname);
+                     int displayname, int expect100, const char *s_charset);
+
+
+/* Does an OPTIONS request to check the server capabilities. In case of
+   success it will set the global variable initialized. If the server
+   does not support locks, it will remove the lockstore and set locks
+   to NULL.
+   path : Path to the root collection.
+   return value : 0 on success or an apropriate error code. */
+int dav_init_connection(const char *path);
 
 
 /* Releases all locks (if possible) and closes the session.
@@ -69,6 +81,15 @@ void dav_init_webdav(const char *scheme, const char *host, int port,
 void dav_close_webdav(void);
 
 
+/* Converts the character set of name from local character set into the
+   server character set.
+   If no server character set is configured, it just returns a copy of name.
+   name : filename encoded with the character set of the current locale.
+   return value : a string with name, encoded in the character set used
+                  by the server. */
+char *dav_cconvert(const char *name);
+
+
 /* Deletes file path on the server.
    On success locks for this file are removed from the lock store.
    path   : Absolute path of the file.
@@ -117,10 +138,19 @@ int dav_get_collection(const char *path, dav_props **props);
    mtime      : Points to the Last-Modified value of the cached version. Will
                 be updated if a new version of the file is retrieved.
                 May be NULL.
+   modified   : Points to a flag that will be set 1 if the file cache_path
+                has been replaced by a new version. May be NULL.
    return value : 0 on success; an appropriate file error code otherwise.
                   Not-Modified counts as success. */
 int dav_get_file(const char *path, const char *cache_path, off_t *size,
-                 char **etag, time_t *mtime);
+                 char **etag, time_t *mtime, int *modified);
+
+
+/* Returns the error string from the last WebDAV request.
+   Note: This will not be usefull in any case, because the last function
+         called may have done more then one request (e.g. an additional
+         lock discover. But it is usefull for dav_get_collection(). */
+const char *dav_get_webdav_error(void);
 
 
 /* Tests for the existence of file path and uptdates etag and mtime.
@@ -193,14 +223,15 @@ int dav_move(const char *src, const char *dst);
                 Updated on success. May be NULL.
    mtime      : The Last_Modified value used to check for changes on the
                 server. Updated on success. May be NULL.
+   execute    : if 1 set execute property, else no change of execute property.
    return value : 0 on success; an appropriate file error code otherwise. */
 int dav_put(const char *path, const char *cache_path, int *exists,
-            time_t *expire, char **etag, time_t *mtime);
+            time_t *expire, char **etag, time_t *mtime, int execute);
 
 
 /* Sets or resets the execute property of file path.
    path : Absolute path of the file on the server.
-   set  : boolean value; 0 reset execute propertie; 1 set execute propertie. */
+   set  : boolean value; 0 reset execute property; 1 set execute property. */
 int dav_set_execute(const char *path, int set);
 
 
